@@ -622,6 +622,12 @@ namespace Magick
     void            shade ( const double azimuth_ = 30,
           const double elevation_ = 30,
           const bool   colorShading_ = false );
+
+    // Simulate an image shadow
+    void            shadow ( const double percent_opacity_ = 80.0,
+                             const double sigma_ = 0.5,
+                             const ssize_t x_ = 5,
+                             const ssize_t y_ = 5 );
     
     // Sharpen pixels in image
     // The radius_ parameter specifies the radius of the Gaussian, in
