@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -191,7 +191,8 @@ static Image *ReadMACImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (bit == 0)
                   byte=(*p++);
-                indexes[x]=(IndexPacket) ((byte & 0x80) != 0 ? 0x01 : 0x00);
+                SetPixelIndex(indexes+x,((byte & 0x80) != 0 ?
+                  0x01 : 0x00));
                 bit++;
                 byte<<=1;
                 if (bit == 8)
@@ -226,7 +227,8 @@ static Image *ReadMACImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             if (bit == 0)
               byte=(*p++);
-            indexes[x]=(IndexPacket) ((byte & 0x80) != 0 ? 0x01 : 0x00);
+            SetPixelIndex(indexes+x,((byte & 0x80) != 0 ?
+              0x01 : 0x00));
             bit++;
             byte<<=1;
             if (bit == 8)
