@@ -372,7 +372,7 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
   */
   stride=(size_t) cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32,
     image->columns);
-  pixels=(unsigned char *) AcquireQuantumMemory(image->columns,stride*
+  pixels=(unsigned char *) AcquireQuantumMemory(image->rows,stride*
     sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     {
@@ -421,7 +421,7 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
         Disassociate alpha.
       */
       gamma=1.0-QuantumScale*fill_color.opacity;
-      gamma=1.0/(fabs((double) gamma) <= MagickEpsilon ? 1.0 : gamma);
+      gamma=MagickEpsilonReciprocal(gamma);
       fill_color.blue*=gamma;
       fill_color.green*=gamma;
       fill_color.red*=gamma;
