@@ -22,7 +22,7 @@
 %                                March 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -220,7 +220,6 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
 
   Image
     *destination_image,
-    *duplex_image,
     *source_image;
 
   MagickBooleanType
@@ -229,6 +228,10 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
   MagickOffsetType
     progress;
 
+  size_t
+    height,
+    width;
+
   ssize_t
     y;
 
@@ -237,15 +240,17 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
   if (transfer == (DuplexTransferImageViewMethod) NULL)
     return(MagickFalse);
   source_image=source->image;
-  duplex_image=duplex->image;
   destination_image=destination->image;
   if (SetImageStorageClass(destination_image,DirectClass) == MagickFalse)
     return(MagickFalse);
   status=MagickTrue;
   progress=0;
+  height=(size_t) (source->extent.height-source->extent.y);
+  width=(size_t) (source->extent.width-source->extent.x);
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status) num_threads(source->number_threads)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    dynamic_number_threads(source_image,width,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
@@ -255,17 +260,10 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
     MagickBooleanType
       sync;
 
-    register const IndexPacket
-      *restrict duplex_indexes,
-      *restrict indexes;
-
     register const PixelPacket
       *restrict duplex_pixels,
       *restrict pixels;
 
-    register IndexPacket
-      *restrict destination_indexes;
-
     register PixelPacket
       *restrict destination_pixels;
 
@@ -278,7 +276,6 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexQueue(source->view);
     duplex_pixels=GetCacheViewVirtualPixels(duplex->view,duplex->extent.x,y,
       duplex->extent.width,1,duplex->exception);
     if (duplex_pixels == (const PixelPacket *) NULL)
@@ -286,7 +283,6 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
         status=MagickFalse;
         continue;
       }
-    duplex_indexes=GetCacheViewVirtualIndexQueue(duplex->view);
     destination_pixels=GetCacheViewAuthenticPixels(destination->view,
       destination->extent.x,y,destination->extent.width,1,exception);
     if (destination_pixels == (PixelPacket *) NULL)
@@ -294,7 +290,6 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
         status=MagickFalse;
         continue;
       }
-    destination_indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     if (transfer(source,duplex,destination,y,id,context) == MagickFalse)
       status=MagickFalse;
     sync=SyncCacheViewAuthenticPixels(destination->view,exception);
@@ -310,7 +305,7 @@ MagickExport MagickBooleanType DuplexTransferImageViewIterator(
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_DuplexTransferImageViewIterator)
+        #pragma omp critical (MagickCore_DuplexTransferImageViewIterator)
 #endif
         proceed=SetImageProgress(source_image,source->description,progress++,
           source->extent.height);
@@ -550,6 +545,10 @@ MagickExport MagickBooleanType GetImageViewIterator(ImageView *source,
   MagickOffsetType
     progress;
 
+  size_t
+    height,
+    width;
+
   ssize_t
     y;
 
@@ -560,17 +559,17 @@ MagickExport MagickBooleanType GetImageViewIterator(ImageView *source,
   source_image=source->image;
   status=MagickTrue;
   progress=0;
+  height=(size_t) (source->extent.height-source->extent.y);
+  width=(size_t) (source->extent.width-source->extent.x);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status) num_threads(source->number_threads)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    dynamic_number_threads(source_image,width,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
     const int
       id = GetOpenMPThreadId();
 
-    register const IndexPacket
-      *indexes;
-
     register const PixelPacket
       *pixels;
 
@@ -583,7 +582,6 @@ MagickExport MagickBooleanType GetImageViewIterator(ImageView *source,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexQueue(source->view);
     if (get(source,y,id,context) == MagickFalse)
       status=MagickFalse;
     if (source_image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -592,7 +590,7 @@ MagickExport MagickBooleanType GetImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_GetImageViewIterator)
+        #pragma omp critical (MagickCore_GetImageViewIterator)
 #endif
         proceed=SetImageProgress(source_image,source->description,progress++,
           source->extent.height);
@@ -732,13 +730,14 @@ MagickExport ImageView *NewImageView(Image *image)
   (void) ResetMagickMemory(image_view,0,sizeof(*image_view));
   image_view->description=ConstantString("ImageView");
   image_view->image=image;
-  image_view->view=AcquireCacheView(image_view->image);
+  image_view->exception=AcquireExceptionInfo();
+  image_view->view=AcquireVirtualCacheView(image_view->image,
+    image_view->exception);
   image_view->extent.width=image->columns;
   image_view->extent.height=image->rows;
   image_view->extent.x=0;
   image_view->extent.y=0;
   image_view->number_threads=GetOpenMPMaximumThreads();
-  image_view->exception=AcquireExceptionInfo();
   image_view->debug=IsEventLogging();
   image_view->signature=MagickSignature;
   return(image_view);
@@ -784,14 +783,15 @@ MagickExport ImageView *NewImageViewRegion(Image *image,const ssize_t x,
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) ResetMagickMemory(image_view,0,sizeof(*image_view));
   image_view->description=ConstantString("ImageView");
-  image_view->view=AcquireCacheView(image_view->image);
+  image_view->exception=AcquireExceptionInfo();
+  image_view->view=AcquireVirtualCacheView(image_view->image,
+    image_view->exception);
   image_view->image=image;
   image_view->extent.width=width;
   image_view->extent.height=height;
   image_view->extent.x=x;
   image_view->extent.y=y;
   image_view->number_threads=GetOpenMPMaximumThreads();
-  image_view->exception=AcquireExceptionInfo();
   image_view->debug=IsEventLogging();
   image_view->signature=MagickSignature;
   return(image_view);
@@ -889,6 +889,10 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
   MagickOffsetType
     progress;
 
+  size_t
+    height,
+    width;
+
   ssize_t
     y;
 
@@ -901,9 +905,12 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
     return(MagickFalse);
   status=MagickTrue;
   progress=0;
+  height=(size_t) (destination->extent.height-destination->extent.y);
+  width=(size_t) (destination->extent.width-destination->extent.x);
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status) num_threads(destination->number_threads)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    dynamic_number_threads(destination_image,width,height,1)
 #endif
   for (y=destination->extent.y; y < (ssize_t) destination->extent.height; y++)
   {
@@ -913,9 +920,6 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
     MagickBooleanType
       sync;
 
-    register IndexPacket
-      *restrict indexes;
-
     register PixelPacket
       *restrict pixels;
 
@@ -930,7 +934,6 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     if (set(destination,y,id,context) == MagickFalse)
       status=MagickFalse;
     sync=SyncCacheViewAuthenticPixels(destination->view,exception);
@@ -946,7 +949,7 @@ MagickExport MagickBooleanType SetImageViewIterator(ImageView *destination,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_SetImageViewIterator)
+        #pragma omp critical (MagickCore_SetImageViewIterator)
 #endif
         proceed=SetImageProgress(destination_image,destination->description,
           progress++,destination->extent.height);
@@ -988,7 +991,7 @@ MagickExport void SetImageViewThreads(ImageView *image_view,
   assert(image_view != (ImageView *) NULL);
   assert(image_view->signature == MagickSignature);
   image_view->number_threads=number_threads;
-  if (number_threads > GetOpenMPMaximumThreads())
+  if (number_threads > (size_t) GetMagickResourceLimit(ThreadResource))
     image_view->number_threads=GetOpenMPMaximumThreads();
 }
 
@@ -1056,6 +1059,10 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
   MagickOffsetType
     progress;
 
+  size_t
+    height,
+    width;
+
   ssize_t
     y;
 
@@ -1069,9 +1076,12 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
     return(MagickFalse);
   status=MagickTrue;
   progress=0;
+  height=(size_t) (source->extent.height-source->extent.y);
+  width=(size_t) (source->extent.width-source->extent.x);
   exception=destination->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status) num_threads(source->number_threads)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    dynamic_number_threads(source_image,width,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
@@ -1081,15 +1091,9 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
     MagickBooleanType
       sync;
 
-    register const IndexPacket
-      *restrict indexes;
-
     register const PixelPacket
       *restrict pixels;
 
-    register IndexPacket
-      *restrict destination_indexes;
-
     register PixelPacket
       *restrict destination_pixels;
 
@@ -1102,7 +1106,6 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexQueue(source->view);
     destination_pixels=GetCacheViewAuthenticPixels(destination->view,
       destination->extent.x,y,destination->extent.width,1,exception);
     if (destination_pixels == (PixelPacket *) NULL)
@@ -1110,7 +1113,6 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
         status=MagickFalse;
         continue;
       }
-    destination_indexes=GetCacheViewAuthenticIndexQueue(destination->view);
     if (transfer(source,destination,y,id,context) == MagickFalse)
       status=MagickFalse;
     sync=SyncCacheViewAuthenticPixels(destination->view,exception);
@@ -1126,7 +1128,7 @@ MagickExport MagickBooleanType TransferImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_TransferImageViewIterator)
+        #pragma omp critical (MagickCore_TransferImageViewIterator)
 #endif
         proceed=SetImageProgress(source_image,source->description,progress++,
           source->extent.height);
@@ -1195,6 +1197,10 @@ MagickExport MagickBooleanType UpdateImageViewIterator(ImageView *source,
   MagickOffsetType
     progress;
 
+  size_t
+    height,
+    width;
+
   ssize_t
     y;
 
@@ -1207,18 +1213,18 @@ MagickExport MagickBooleanType UpdateImageViewIterator(ImageView *source,
     return(MagickFalse);
   status=MagickTrue;
   progress=0;
+  height=(size_t) (source->extent.height-source->extent.y);
+  width=(size_t) (source->extent.width-source->extent.x);
   exception=source->exception;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,1) shared(progress,status) num_threads(source->number_threads)
+  #pragma omp parallel for schedule(static) shared(progress,status) \
+    dynamic_number_threads(source_image,width,height,1)
 #endif
   for (y=source->extent.y; y < (ssize_t) source->extent.height; y++)
   {
     const int
       id = GetOpenMPThreadId();
 
-    register IndexPacket
-      *restrict indexes;
-
     register PixelPacket
       *restrict pixels;
 
@@ -1232,7 +1238,6 @@ MagickExport MagickBooleanType UpdateImageViewIterator(ImageView *source,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexQueue(source->view);
     if (update(source,y,id,context) == MagickFalse)
       status=MagickFalse;
     if (SyncCacheViewAuthenticPixels(source->view,exception) == MagickFalse)
@@ -1246,7 +1251,7 @@ MagickExport MagickBooleanType UpdateImageViewIterator(ImageView *source,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_UpdateImageViewIterator)
+        #pragma omp critical (MagickCore_UpdateImageViewIterator)
 #endif
         proceed=SetImageProgress(source_image,source->description,progress++,
           source->extent.height);
