@@ -1,12 +1,12 @@
 /*
   Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
-  
+
   You may not use this file except in compliance with the License.
   obtain a copy of the License at
-  
+
     http://www.imagemagick.org/script/license.php
-  
+
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,25 +22,11 @@
 extern "C" {
 #endif
 
-#include <magick/exception-private.h>
-#include <magick/image.h>
-#include <magick/color.h>
-#include <magick/image-private.h>
-#include <magick/quantum-private.h>
-
-static inline MagickPixelPacket *CloneMagickPixelPacket(
-  const MagickPixelPacket *pixel)
-{
-  MagickPixelPacket
-    *clone_pixel;
-
-  clone_pixel=(MagickPixelPacket *) AcquireAlignedMemory(1,
-    sizeof(*clone_pixel));
-  if (clone_pixel == (MagickPixelPacket *) NULL)
-    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  *clone_pixel=(*pixel);
-  return(clone_pixel);
-}
+#include "magick/image.h"
+#include "magick/color.h"
+#include "magick/image-private.h"
+#include "magick/memory_.h"
+#include "magick/quantum-private.h"
 
 static inline MagickBooleanType IsGrayPixel(const PixelPacket *pixel)
 {
@@ -56,7 +42,7 @@ static inline MagickBooleanType IsGrayPixel(const PixelPacket *pixel)
 
     alpha=GetPixelRed(pixel)-GetPixelGreen(pixel);
     beta=GetPixelGreen(pixel)-GetPixelBlue(pixel);
-    if ((fabs(alpha) <= MagickEpsilon) && (fabs(beta) <= MagickEpsilon))
+    if ((fabs((double) alpha) <= MagickEpsilon) && (fabs(beta) <= MagickEpsilon))
       return(MagickTrue);
   }
 #endif
@@ -67,7 +53,7 @@ static inline MagickBooleanType IsMonochromePixel(const PixelPacket *pixel)
 {
 #if !defined(MAGICKCORE_HDRI_SUPPORT)
   if (((GetPixelRed(pixel) == 0) ||
-       (GetPixelRed(pixel) == (Quantum) QuantumRange)) &&
+       (GetPixelRed(pixel) == QuantumRange)) &&
       (GetPixelRed(pixel) == GetPixelGreen(pixel)) &&
       (GetPixelGreen(pixel) == GetPixelBlue(pixel)))
     return(MagickTrue);
@@ -81,13 +67,35 @@ static inline MagickBooleanType IsMonochromePixel(const PixelPacket *pixel)
     beta=GetPixelGreen(pixel)-GetPixelBlue(pixel);
     if (((fabs(GetPixelRed(pixel)) <= MagickEpsilon) ||
          (fabs(GetPixelRed(pixel)-QuantumRange) <= MagickEpsilon)) &&
-        (fabs(alpha) <= MagickEpsilon) && (fabs(beta) <= MagickEpsilon))
+        (fabs((double) alpha) <= MagickEpsilon) && (fabs(beta) <= MagickEpsilon))
       return(MagickTrue);
     }
 #endif
   return(MagickFalse);
 }
 
+static inline MagickRealType MagickEpsilonReciprocal(const MagickRealType x)
+{
+  MagickRealType sign = x < (MagickRealType) 0.0 ? (MagickRealType) -1.0 : 
+    (MagickRealType) 1.0;
+  return((sign*x) > MagickEpsilon ? (MagickRealType) 1.0/x : sign*(
+    (MagickRealType) 1.0/MagickEpsilon));
+}
+
+static inline double CompandsRGB(const double intensity)
+{
+  if (intensity <= 0.0031308)
+    return(intensity*12.92);
+  return(1.055*pow(intensity,1.0/2.4)-0.055);
+}
+
+static inline double DecompandsRGB(const double intensity)
+{
+  if (intensity <= 0.04045)
+    return(intensity/12.92);
+  return(pow((intensity+0.055)/1.055,2.4));
+}
+
 static inline void SetMagickPixelPacket(const Image *image,
   const PixelPacket *color,const IndexPacket *index,MagickPixelPacket *pixel)
 {
