@@ -80,6 +80,7 @@
 #include "magick/quantize.h"
 #include "magick/random_.h"
 #include "magick/random-private.h"
+#include "magick/resource_.h"
 #include "magick/segment.h"
 #include "magick/semaphore.h"
 #include "magick/signature-private.h"
@@ -138,7 +139,7 @@ static MagickPixelPacket **DestroyPixelThreadSet(MagickPixelPacket **pixels)
     i;
 
   assert(pixels != (MagickPixelPacket **) NULL);
-  for (i=0; i < (ssize_t) GetOpenMPMaximumThreads(); i++)
+  for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
     if (pixels[i] != (MagickPixelPacket *) NULL)
       pixels[i]=(MagickPixelPacket *) RelinquishMagickMemory(pixels[i]);
   pixels=(MagickPixelPacket **) RelinquishMagickMemory(pixels);
@@ -298,8 +299,9 @@ static MagickRealType ApplyEvaluateOperator(RandomInfo *random_info,
     }
     case LogEvaluateOperator:
     {
-      result=(MagickRealType) (QuantumRange*log((double) (QuantumScale*value*
-        pixel+1.0))/log((double) (value+1.0)));
+      if ((QuantumScale*pixel) >= MagickEpsilon)
+        result=(MagickRealType) (QuantumRange*log((double) (QuantumScale*value*
+          pixel+1.0))/log((double) (value+1.0)));
       break;
     }
     case MaxEvaluateOperator:
@@ -430,7 +432,7 @@ MagickExport Image *EvaluateImages(const Image *images,
     *next;
 
   Image
-    *evaluate_image;
+    *image;
 
   MagickBooleanType
     status;
@@ -451,6 +453,9 @@ MagickExport Image *EvaluateImages(const Image *images,
   ssize_t
     y;
 
+  unsigned long
+    key;
+
   /*
     Ensure the image are the same size.
   */
@@ -470,21 +475,20 @@ MagickExport Image *EvaluateImages(const Image *images,
   /*
     Initialize evaluate next attributes.
   */
-  evaluate_image=CloneImage(images,images->columns,images->rows,MagickTrue,
-    exception);
-  if (evaluate_image == (Image *) NULL)
+  image=CloneImage(images,images->columns,images->rows,MagickTrue,exception);
+  if (image == (Image *) NULL)
     return((Image *) NULL);
-  if (SetImageStorageClass(evaluate_image,DirectClass) == MagickFalse)
+  if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     {
-      InheritException(exception,&evaluate_image->exception);
-      evaluate_image=DestroyImage(evaluate_image);
+      InheritException(exception,&image->exception);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   number_images=GetImageListLength(images);
   evaluate_pixels=AcquirePixelThreadSet(images,number_images);
   if (evaluate_pixels == (MagickPixelPacket **) NULL)
     {
-      evaluate_image=DestroyImage(evaluate_image);
+      image=DestroyImage(image);
       (void) ThrowMagickException(exception,GetMagickModule(),
         ResourceLimitError,"MemoryAllocationFailed","`%s'",images->filename);
       return((Image *) NULL);
@@ -496,13 +500,15 @@ MagickExport Image *EvaluateImages(const Image *images,
   progress=0;
   GetMagickPixelPacket(images,&zero);
   random_info=AcquireRandomInfoThreadSet();
-  evaluate_view=AcquireCacheView(evaluate_image);
+  key=GetRandomSecretKey(random_info[0]);
+  evaluate_view=AcquireAuthenticCacheView(image,exception);
   if (op == MedianEvaluateOperator)
     {
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static) shared(progress,status)
+      #pragma omp parallel for schedule(static) shared(progress,status) \
+        dynamic_number_threads(image,image->columns,image->rows,key == ~0UL)
 #endif
-      for (y=0; y < (ssize_t) evaluate_image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
         CacheView
           *image_view;
@@ -528,7 +534,7 @@ MagickExport Image *EvaluateImages(const Image *images,
         if (status == MagickFalse)
           continue;
         q=QueueCacheViewAuthenticPixels(evaluate_view,0,y,
-          evaluate_image->columns,1,exception);
+          image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
@@ -536,7 +542,7 @@ MagickExport Image *EvaluateImages(const Image *images,
           }
         evaluate_indexes=GetCacheViewAuthenticIndexQueue(evaluate_view);
         evaluate_pixel=evaluate_pixels[id];
-        for (x=0; x < (ssize_t) evaluate_image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
           register ssize_t
             i;
@@ -552,7 +558,7 @@ MagickExport Image *EvaluateImages(const Image *images,
             register const PixelPacket
               *p;
 
-            image_view=AcquireCacheView(next);
+            image_view=AcquireVirtualCacheView(next,exception);
             p=GetCacheViewVirtualPixels(image_view,x,y,1,1,exception);
             if (p == (const PixelPacket *) NULL)
               {
@@ -568,7 +574,7 @@ MagickExport Image *EvaluateImages(const Image *images,
               GetPixelBlue(p),op,evaluate_pixel[i].blue);
             evaluate_pixel[i].opacity=ApplyEvaluateOperator(random_info[id],
               GetPixelOpacity(p),op,evaluate_pixel[i].opacity);
-            if (evaluate_image->colorspace == CMYKColorspace)
+            if (image->colorspace == CMYKColorspace)
               evaluate_pixel[i].index=ApplyEvaluateOperator(random_info[id],
                 *indexes,op,evaluate_pixel[i].index);
             image_view=DestroyCacheView(image_view);
@@ -579,12 +585,12 @@ MagickExport Image *EvaluateImages(const Image *images,
           SetPixelRed(q,ClampToQuantum(evaluate_pixel[i/2].red));
           SetPixelGreen(q,ClampToQuantum(evaluate_pixel[i/2].green));
           SetPixelBlue(q,ClampToQuantum(evaluate_pixel[i/2].blue));
-          if (evaluate_image->matte == MagickFalse)
+          if (image->matte == MagickFalse)
             SetPixelOpacity(q,ClampToQuantum(
               evaluate_pixel[i/2].opacity));
           else
             SetPixelAlpha(q,ClampToQuantum(evaluate_pixel[i/2].opacity));
-          if (evaluate_image->colorspace == CMYKColorspace)
+          if (image->colorspace == CMYKColorspace)
             SetPixelIndex(evaluate_indexes+i,ClampToQuantum(
               evaluate_pixel[i/2].index));
           q++;
@@ -600,7 +606,7 @@ MagickExport Image *EvaluateImages(const Image *images,
             #pragma omp critical (MagickCore_EvaluateImages)
 #endif
             proceed=SetImageProgress(images,EvaluateImageTag,progress++,
-              evaluate_image->rows);
+              image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
@@ -609,9 +615,10 @@ MagickExport Image *EvaluateImages(const Image *images,
   else
     {
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static) shared(progress,status)
+      #pragma omp parallel for schedule(static) shared(progress,status) \
+        dynamic_number_threads(image,image->columns,image->rows,key == ~0UL)
 #endif
-      for (y=0; y < (ssize_t) evaluate_image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
         CacheView
           *image_view;
@@ -638,7 +645,7 @@ MagickExport Image *EvaluateImages(const Image *images,
         if (status == MagickFalse)
           continue;
         q=QueueCacheViewAuthenticPixels(evaluate_view,0,y,
-          evaluate_image->columns,1,exception);
+          image->columns,1,exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
@@ -646,7 +653,7 @@ MagickExport Image *EvaluateImages(const Image *images,
           }
         evaluate_indexes=GetCacheViewAuthenticIndexQueue(evaluate_view);
         evaluate_pixel=evaluate_pixels[id];
-        for (x=0; x < (ssize_t) evaluate_image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
           evaluate_pixel[x]=zero;
         next=images;
         for (i=0; i < (ssize_t) number_images; i++)
@@ -657,7 +664,7 @@ MagickExport Image *EvaluateImages(const Image *images,
           register const PixelPacket
             *p;
 
-          image_view=AcquireCacheView(next);
+          image_view=AcquireVirtualCacheView(next,exception);
           p=GetCacheViewVirtualPixels(image_view,0,y,next->columns,1,exception);
           if (p == (const PixelPacket *) NULL)
             {
@@ -679,7 +686,7 @@ MagickExport Image *EvaluateImages(const Image *images,
             evaluate_pixel[x].opacity=ApplyEvaluateOperator(random_info[id],
               GetPixelOpacity(p),i == 0 ? AddEvaluateOperator : op,
               evaluate_pixel[x].opacity);
-            if (evaluate_image->colorspace == CMYKColorspace)
+            if (image->colorspace == CMYKColorspace)
               evaluate_pixel[x].index=ApplyEvaluateOperator(random_info[id],
                 GetPixelIndex(indexes+x),i == 0 ? AddEvaluateOperator : op,
                 evaluate_pixel[x].index);
@@ -689,7 +696,7 @@ MagickExport Image *EvaluateImages(const Image *images,
           next=GetNextImageInList(next);
         }
         if (op == MeanEvaluateOperator)
-          for (x=0; x < (ssize_t) evaluate_image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             evaluate_pixel[x].red/=number_images;
             evaluate_pixel[x].green/=number_images;
@@ -698,7 +705,7 @@ MagickExport Image *EvaluateImages(const Image *images,
             evaluate_pixel[x].index/=number_images;
           }
         if (op == MultiplyEvaluateOperator)
-          for (x=0; x < (ssize_t) evaluate_image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
             register ssize_t
               j;
@@ -712,16 +719,16 @@ MagickExport Image *EvaluateImages(const Image *images,
               evaluate_pixel[x].index*=QuantumScale;
             }
           }
-        for (x=0; x < (ssize_t) evaluate_image->columns; x++)
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
           SetPixelRed(q,ClampToQuantum(evaluate_pixel[x].red));
           SetPixelGreen(q,ClampToQuantum(evaluate_pixel[x].green));
           SetPixelBlue(q,ClampToQuantum(evaluate_pixel[x].blue));
-          if (evaluate_image->matte == MagickFalse)
+          if (image->matte == MagickFalse)
             SetPixelOpacity(q,ClampToQuantum(evaluate_pixel[x].opacity));
           else
             SetPixelAlpha(q,ClampToQuantum(evaluate_pixel[x].opacity));
-          if (evaluate_image->colorspace == CMYKColorspace)
+          if (image->colorspace == CMYKColorspace)
             SetPixelIndex(evaluate_indexes+x,ClampToQuantum(
               evaluate_pixel[x].index));
           q++;
@@ -737,7 +744,7 @@ MagickExport Image *EvaluateImages(const Image *images,
             #pragma omp critical (MagickCore_EvaluateImages)
 #endif
             proceed=SetImageProgress(images,EvaluateImageTag,progress++,
-              evaluate_image->rows);
+              image->rows);
             if (proceed == MagickFalse)
               status=MagickFalse;
           }
@@ -747,8 +754,8 @@ MagickExport Image *EvaluateImages(const Image *images,
   evaluate_pixels=DestroyPixelThreadSet(evaluate_pixels);
   random_info=DestroyRandomInfoThreadSet(random_info);
   if (status == MagickFalse)
-    evaluate_image=DestroyImage(evaluate_image);
-  return(evaluate_image);
+    image=DestroyImage(image);
+  return(image);
 }
 
 MagickExport MagickBooleanType EvaluateImageChannel(Image *image,
@@ -770,6 +777,9 @@ MagickExport MagickBooleanType EvaluateImageChannel(Image *image,
   ssize_t
     y;
 
+  unsigned long
+    key;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -784,9 +794,11 @@ MagickExport MagickBooleanType EvaluateImageChannel(Image *image,
   status=MagickTrue;
   progress=0;
   random_info=AcquireRandomInfoThreadSet();
-  image_view=AcquireCacheView(image);
+  key=GetRandomSecretKey(random_info[0]);
+  image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,key == ~0UL)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -844,7 +856,7 @@ MagickExport MagickBooleanType EvaluateImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_EvaluateImageChannel)
+        #pragma omp critical (MagickCore_EvaluateImageChannel)
 #endif
         proceed=SetImageProgress(image,EvaluateImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
@@ -1022,9 +1034,10 @@ MagickExport MagickBooleanType FunctionImageChannel(Image *image,
     }
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheView(image);
+  image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
   {
@@ -1079,7 +1092,7 @@ MagickExport MagickBooleanType FunctionImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_FunctionImageChannel)
+        #pragma omp critical (MagickCore_FunctionImageChannel)
 #endif
         proceed=SetImageProgress(image,FunctionImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
@@ -1958,7 +1971,7 @@ static PixelList **DestroyPixelListThreadSet(PixelList **pixel_list)
     i;
 
   assert(pixel_list != (PixelList **) NULL);
-  for (i=0; i < (ssize_t) GetOpenMPMaximumThreads(); i++)
+  for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
     if (pixel_list[i] != (PixelList *) NULL)
       pixel_list[i]=DestroyPixelList(pixel_list[i]);
   pixel_list=(PixelList **) RelinquishMagickMemory(pixel_list);
@@ -2551,10 +2564,11 @@ MagickExport Image *StatisticImageChannel(const Image *image,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheView(image);
-  statistic_view=AcquireCacheView(statistic_image);
+  image_view=AcquireVirtualCacheView(image,exception);
+  statistic_view=AcquireAuthenticCacheView(statistic_image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(progress,status)
+  #pragma omp parallel for schedule(static,4) shared(progress,status) \
+    dynamic_number_threads(image,image->columns,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) statistic_image->rows; y++)
   {
@@ -2696,7 +2710,7 @@ MagickExport Image *StatisticImageChannel(const Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical (MagickCore_StatisticImage)
+        #pragma omp critical (MagickCore_StatisticImage)
 #endif
         proceed=SetImageProgress(image,StatisticImageTag,progress++,
           image->rows);
