@@ -18,7 +18,7 @@
 %                            December 2003                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2011 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -59,7 +59,7 @@
 %
 */
 
-int CompareMain(int argc,char **argv)
+static int CompareMain(int argc,char **argv)
 {
   char
     *metadata;
