@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2012 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -22,9 +22,6 @@
 extern "C" {
 #endif
 
-extern WandExport void
-  PixelGetMagickColor(const PixelWand *,MagickPixelPacket *);
-
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
